//============================================================================
// Hierarchical state machine manager
//
// Timer.c : generic timer routines
//
// (c) Robert Lacoste 2013, 
//
// Use this software freely, but provided without any warranty or support
//============================================================================

#include "Timer.h"
#include "Lowlevel.h"


//----------------------------------------------------------------------------
// Timer global variables
//----------------------------------------------------------------------------

// Internal structure of a timer
typedef struct 
{
	unsigned long TimerValue;	// Remaining value (in milliseconds)
	HSM_T_HSMID HsmId;			// Owner HSM (or HSM_ID_NOTAFFECTED)
}
T_TIMER;

// Timers array
T_TIMER Timers[TIMER_COUNT];


//----------------------------------------------------------------------------
// TimerInit : Initilaisation of the timers
//----------------------------------------------------------------------------

void TimerInit(void)
{
	int i;
	for(i=0;i<TIMER_COUNT;i++)
	{
		Timers[i].TimerValue=0L;
		Timers[i].HsmId=HSM_ID_NOTAFFECTED;
	}
}


//----------------------------------------------------------------------------
// TimerTick1s : Should be called once per second (from interrupt or 
// from idle function)
//----------------------------------------------------------------------------

BOOL TimerTick1s(void)
{
	int i;
	BOOL eventfound=FALSE;
	
	for(i=0;i<TIMER_COUNT;i++)
	{
		// Check if a timer should be decremented
		if(Timers[i].TimerValue!=0L)
		{
			if(Timers[i].TimerValue>TIMER_VAL_1S)
				Timers[i].TimerValue-=TIMER_VAL_1S;
			else
				Timers[i].TimerValue=0L;
		}
		
		// Check if a timer is active has just ended
		if((Timers[i].HsmId!=HSM_ID_NOTAFFECTED)&&(Timers[i].TimerValue==0L))
		{
			// If so post a TIMEOUT message to the corresponding HSM
			HsmSendMsg(Timers[i].HsmId,TIMER_TIMEOUT_T1+i,0);
			Timers[i].HsmId=HSM_ID_NOTAFFECTED;
			eventfound=TRUE;
		}
	}
	
	return(eventfound);
}


//----------------------------------------------------------------------------
// TimerArm : Set and arm a given timer
//----------------------------------------------------------------------------

void TimerArm(T_ID_TIMER tid, HSM_T_HSMID hsm_id, unsigned long val_ms)
{
	assert(tid<TIMER_COUNT);
	assert(hsm_id<HSM_PER_SYSTEM);
	Timers[tid].TimerValue=val_ms;
	Timers[tid].HsmId=hsm_id;
}


//----------------------------------------------------------------------------
// TimerStop : Stop a running timer
//----------------------------------------------------------------------------

void TimerStop(T_ID_TIMER tid)
{
	Timers[tid].TimerValue=0L;
	Timers[tid].HsmId=HSM_ID_NOTAFFECTED;
}


//----------------------------------------------------------------------------
// TimerIsEnded : Check (polling) a given timer
//----------------------------------------------------------------------------

BOOL TimerIsEnded(T_ID_TIMER tid)
{
	return(Timers[tid].TimerValue==0L);
}

// END OF FILE


