//============================================================================
// Hierarchical state machine manager
//
// HSM.C : HSM management processor
//
// (c) Robert Lacoste 2013, 
// Based on ideas from Miro Samek's book "Practival UML statecharts in C/C++)"
//
// Use this software freely, but provided without any warranty or support
//============================================================================


#include "HSM.h"
#include "LowLevel.h"

//----------------------------------------------------------------------------
// HSM global variables
//----------------------------------------------------------------------------

// Message queue (one queue for each HSM of the system)
HSM_T_SIGNAL HsmSignalQueue[HSM_PER_SYSTEM][HSM_MSG_QUEUE_LENGTH];

// Pointers on the message queue occupied areas (FIFO)
unsigned short HsmSignalQueueFirstFreePos[HSM_PER_SYSTEM];
unsigned short HsmSignalQueueFirstUsedPos[HSM_PER_SYSTEM];

// Pointers to the current state handling function for each HSM
HSM_T_STATEHANDLINGFUNCTION HsmCurrentState[HSM_PER_SYSTEM];

// Priotity of each HSM
HSM_T_PRIORITY HsmPriority[HSM_PER_SYSTEM];

// Pointer to the idle function
HSM_T_IDLEFUNCTION HsmIdleFunction;


//----------------------------------------------------------------------------
// HsmInit : Initialisation of the HSM processor
//----------------------------------------------------------------------------

void HsmInit(void)
{
	int i;
	
	// Just an initialisation of all global memory
	for(i=0;i<HSM_PER_SYSTEM;i++)
	{
		HsmSignalQueueFirstFreePos[i]=0;
		HsmSignalQueueFirstUsedPos[i]=0;
		HsmCurrentState[i]=(HSM_T_STATEHANDLINGFUNCTION)0;
		HsmPriority[i]=HSM_UNUSED;
	}
	HsmIdleFunction=(HSM_T_IDLEFUNCTION)0;
}


//----------------------------------------------------------------------------
// HsmDefineIdleFunction : Define a pointer to the idle function
//----------------------------------------------------------------------------

void HsmDefineIdleFunction(HSM_T_IDLEFUNCTION idle_function)
{
	// Just store the pointer
	assert(idle_function!=0);
	HsmIdleFunction=idle_function;
}


//----------------------------------------------------------------------------
// HsmSendMsg : Post a message on the message queue of a given HSM
//----------------------------------------------------------------------------

void HsmSendMsg(HSM_T_HSMID hsm_id,HSM_T_SIGNALID signal_id,HSM_T_SIGNALPARAMETER parameter)
{
	unsigned char p;
	
	assert(hsm_id<HSM_PER_SYSTEM);
	// Just store the message and update the FIFO pointers
	disable_interrupts(); 	//==========PROTECTED====================
	p=(HsmSignalQueueFirstFreePos[hsm_id]+1)%HSM_MSG_QUEUE_LENGTH;
	assert(p!=HsmSignalQueueFirstUsedPos[hsm_id]);
	HsmSignalQueue[hsm_id][HsmSignalQueueFirstFreePos[hsm_id]].SignalId=signal_id;
	HsmSignalQueue[hsm_id][HsmSignalQueueFirstFreePos[hsm_id]].SignalParameter=parameter;
	HsmSignalQueueFirstFreePos[hsm_id]=p;
	enable_interrupts(); //===========ENDPROTECTED================

}


//----------------------------------------------------------------------------
// HsmSeetNewState : Move a given HSM to a new state
//
// The new state is identified by a pointer to the state handling function
//----------------------------------------------------------------------------

void HsmSetNewState(HSM_T_HSMID hsm_id,HSM_T_STATEHANDLINGFUNCTION NewState)
{
	HSM_T_SIGNAL sig;
		
	assert(hsm_id<HSM_PER_SYSTEM);
	assert(NewState!=(HSM_T_STATEHANDLINGFUNCTION)0);
	
	// Execute the EXIT actions of the current state if it is not the startup
	if(HsmCurrentState[hsm_id]!=(HSM_T_STATEHANDLINGFUNCTION)0)
	{
		sig.SignalId=HSM_ON_EXIT;
		sig.SignalParameter=0;
		(*HsmCurrentState[hsm_id])(sig);
	}
	
	// Store the pointer to the new state handling function
	HsmCurrentState[hsm_id]=NewState;
	
	// Execute the ENTRY actions of the new state
	sig.SignalId=HSM_ON_ENTRY;
	sig.SignalParameter=0;
	(*HsmCurrentState[hsm_id])(sig);
}


//----------------------------------------------------------------------------
// HsmRun : Run the overall state machine processor
//----------------------------------------------------------------------------
			
void HsmRun(void)
{
	signed char hsm_to_proceed;
	HSM_T_PRIORITY priority;
	unsigned char i;
	
	// Never returns
	while(TRUE)
	{
		// dispatch messages
		do
		{
			hsm_to_proceed=-1;;
			
			// Look for an HSM with a non empty message queue, starting with high priority HSM's
			for(priority=HSM_PRIORITY_HIGH; (priority<=HSM_PRIORITY_LOW)&&(hsm_to_proceed<0); priority++)
			{
				for(i=0;(i<HSM_PER_SYSTEM)&&(hsm_to_proceed<0);i++)
					if(HsmPriority[i]=priority)
						if(HsmSignalQueueFirstFreePos[i]!=HsmSignalQueueFirstUsedPos[i])
							hsm_to_proceed=i;
			}
			
			if(hsm_to_proceed>=0)
			{
				// Execute HSM
				assert(HsmCurrentState[hsm_to_proceed]!=(HSM_T_STATEHANDLINGFUNCTION)0);
				(*HsmCurrentState[hsm_to_proceed])(HsmSignalQueue[hsm_to_proceed][HsmSignalQueueFirstUsedPos[hsm_to_proceed]]);
			
				// Delete message from the queue
				disable_interrupts();  //============PROTECTED==================
				HsmSignalQueueFirstUsedPos[hsm_to_proceed]=(HsmSignalQueueFirstUsedPos[hsm_to_proceed]+1)%HSM_MSG_QUEUE_LENGTH;
				enable_interrupts();	//===========ENDPROTECTED===============
			}
		}
		while(hsm_to_proceed>=0);
		
		// Call idle management function if it is defined
		if(HsmIdleFunction!=(HSM_T_IDLEFUNCTION)0)
			(*HsmIdleFunction)();
	}	
}

// END OF FILE


